# control.py
# Copyright (C) 2002 Evgeny Chukreev <codedj@echo.ru>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#   $Id: control.py,v 1.3 2002/05/14 14:16:59 sjah Exp $
#

"""Base control over alsaplayer.
This module provides widgets to execute base control over alsaplayer.
Also there are two aliases for high level widgets: HControl and VControl.
"""
 
__author__ = "Evgeny Chukreev <codedj@echo.ru>"
__date__ = "$Date: 2002/05/14 14:16:59 $"
__version__ = "$Revision: 1.3 $"
__license__ = "GNU"
 
import aptk.misc, alsaplayer, gtk

# - - - - - - - - - - - - - - - Buttons - - - - - - - - -
class _Button (aptk.misc.Button):
    """Internal."""
    
    def __init__ (self, pl, **keys):
	aptk.misc.Button.__init__ (self, **keys)
	self.connect ("clicked", self._cb_clicked)
	self._pl = pl

class PlayButton (_Button):
    """Start playback.
    
    This class takes playlist object as first argument.
    Optional arguments are same as for
    "misc.Button":misc_Button.py.html#__init__.
    
    Example::

	button = PlayButton (alsaplayer.get_playlist(), label="Play")
    """
    
    def _cb_clicked (self, w):
	self._pl.play (self._pl.get_current ())

class NextButton (_Button):
    """Move to next song.

    This class takes playlist object as first argument.
    Optional arguments are same as for
    "misc.Button":misc_Button.py.html#__init__.

    Example::

	button = NextButton (alsaplayer.get_playlist(), label="Next")
    """
    
    def _cb_clicked (self, w):
	self._pl.next ()

class PrevButton (_Button):
    """Move to previous song.

    This class takes playlist object as first argument.
    Optional arguments are same as for
    "misc.Button":misc_Button.py.html#__init__.

    Example::

	button = PrevButton (alsaplayer.get_playlist(), label="Prev")
    """
    
    def _cb_clicked (self, w):
	self._pl.prev ()

class StopButton (_Button):
    """Stop playback.
    
    This class takes playlist object as first argument.
    Optional arguments are same as for
    "misc.Button":misc_Button.py.html#__init__.
    
    Example::

	button = StopButton (alsaplayer.get_playlist(), label="Stop")
    """
   
    def _cb_clicked (self, w):
	self._pl.stop ()

# - - - - - - - - - - - - - - - Boxes - - - - - - - - - -
class _Box:
    """Internal."""

    def __init__ (self, pl, prev_label=None, prev_xpmfile=None,
			    play_label=None, play_xpmfile=None,
			    stop_label=None, stop_xpmfile=None,
			    next_label=None, next_xpmfile=None, **keys):

	self.prev_button = PrevButton (pl, label=prev_label, xpmfile=prev_xpmfile, **keys)
	self.add (self.prev_button)

	self.play_button = PlayButton (pl, label=play_label, xpmfile=play_xpmfile, **keys)
	self.add (self.play_button)

	self.stop_button = StopButton (pl, label=stop_label, xpmfile=stop_xpmfile, **keys)
	self.add (self.stop_button)

	self.next_button = NextButton (pl, label=next_label, xpmfile=next_xpmfile, **keys)
	self.add (self.next_button)

class HBox (gtk.GtkHBox, _Box):
    """Horizontal box of all control buttons.
    This widget provided to create typical control panel.
    
    **Example**::

	box = HBox (alsaplayer.get_playlist(), prev_label="Prev",
					       play_label="Play",
					       stop_label="Stop",
					       next_label="Next")
    """
    
    def __init__ (self, pl, prev_label=None, prev_xpmfile=None,
			    play_label=None, play_xpmfile=None,
			    stop_label=None, stop_xpmfile=None,
			    next_label=None, next_xpmfile=None, **keys):
	"""Initialize the HBox.

	Parameters:

	    'prev_label' -- text label for button which sets previous song as current.
	
	    'prev_xpmfile' -- xpm file name for button which sets previous song as current.
	
	    'play_label' -- text label for button which starts playback.
	
	    'play_xpmfile' -- xpm file name for button which starts playback.
	
	    'stop_label' -- text label for button which stops playback.
	
	    'stop_xpmfile' -- xpm file name for button which stops playback.
	
	    'next_label' -- text label for button which sets next song as current.
	
	    'next_xpmfile' -- xpm file name for button which sets next song as current.
	
	    '**keys' -- additional, optional values for
			"misc.Button":misc_Button.py.html#__init__.
	"""

	gtk.GtkHBox.__init__ (self)
	_Box.__init__ (self, pl, prev_label=prev_label, prev_xpmfile=prev_xpmfile,
				 play_label=play_label, play_xpmfile=play_xpmfile,
				 stop_label=stop_label, stop_xpmfile=stop_xpmfile,
				 next_label=next_label, next_xpmfile=next_xpmfile,
				 **keys)

class VBox (gtk.GtkVBox, _Box):
    """Vertical box of all control buttons.
    This widget provided to create typical control panel.
    
    **Example**::

	box = VBox (alsaplayer.get_playlist(), prev_label="Prev",
					       play_label="Play",
					       stop_label="Stop",
					       next_label="Next")
    """

    def __init__ (self, pl, prev_label=None, prev_xpmfile=None,
			    play_label=None, play_xpmfile=None,
			    stop_label=None, stop_xpmfile=None,
			    next_label=None, next_xpmfile=None, **keys):
	"""Initialize the VBox.

	Parameters::
    
	    'prev_label' -- text label for button which sets previous song as current.
	
	    'prev_xpmfile' -- xpm file name for button which sets previous song as current.
	
	    'play_label' -- text label for button which starts playback.
	
	    'play_xpmfile' -- xpm file name for button which starts playback.
	
	    'stop_label' -- text label for button which stops playback.
	
	    'stop_xpmfile' -- xpm file name for button which stops playback.
	
	    'next_label' -- text label for button which sets next song as current.
	
	    'next_xpmfile' -- xpm file name for button which sets next song as current.
	
	    '**keys' -- additional, optional values for
			"misc.Button":misc_Button.py.html#__init__.
	"""

	gtk.GtkVBox.__init__ (self)
	_Box.__init__ (self, pl, prev_label=prev_label, prev_xpmfile=prev_xpmfile,
				 play_label=play_label, play_xpmfile=play_xpmfile,
				 stop_label=stop_label, stop_xpmfile=stop_xpmfile,
				 next_label=next_label, next_xpmfile=next_xpmfile,
				 **keys)


# - - - - - - - - - - - -  Final (High level) Widgets
HControl = HBox
VControl = VBox
