/*  pyplaylist.cpp
 *  Copyright (C) 2002 Evgeny Chukreev <codedj@echo.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: pyplaylist.cpp,v 1.4 2002/04/30 00:26:10 sjah Exp $
 *
*/ 

#include "pyplaylist.h"
#include "pyplayitem.h"
#include "pycoreplayer.h"
#include "pyplaylistinterface.h"
#include "alsaplayermodule.h"

#define OBJECT PyPlaylist_Object
#include "pydefs.h"

// Methods:
// C++ name			Python name
// --------------------------------------------------
// Stop				stop
// PlayFile			play_file
// CanPlay			can_play
// GetCorePlayer		get_coreplayer
// Length			get_length
// Play				play
// Next				next
// Prev				prev
// GetCurrent			get_current
// Insert list			insert
// AddAndPlay list		add_and_play
// Remove			remove
// Shuffle			shuffle
// Clear			clear
// Paused			is_paused
// Pause			pause
// UnPause			unpause
// Crossfading			is_crossfading
// Crossfade			crossfade
// UnCrossfade			uncrossfade
// LoopingSong			is_looping_song
// LoopSong			loop_song
// UnLoopSong			unloop_song
// LoopingPlaylist		is_looping_playlist
// LoopPlaylist			loop_playlist
// UnLoopPlaylist		unloop_playlist
// Save				save
// Load				load
// Register			register
// UnRegister			unregister

// TYPE -------------------- ---------------- -------------------
staticforward PyObject *getattr (PyObject *self, char *name);
staticforward void dealloc (PyPlaylist_Object *self);

PyTypeObject PyPlaylist_Type = {
    PyObject_HEAD_INIT (&PyType_Type)
    0,
    "Playlist",
    sizeof (PyPlaylist_Object),
    0,
    (destructor) dealloc,
    0,
    (getattrfunc) getattr,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
};

// MAP by DEFINITIONS ---------------------------------------------------
mapsimple (Stop,		    stop)
mapsimple (Pause,		    pause)
mapsimple (Crossfade,		    crossfade)
mapsimple (LoopSong,		    loop_song)
mapsimple (LoopPlaylist,	    loop_playlist)
mapsimple (UnPause,		    unpause)
mapsimple (UnCrossfade,		    uncrossfade)
mapsimple (UnLoopSong,		    unloop_song)
mapsimple (UnLoopPlaylist,	    unloop_playlist)

mapsimple_with_locking (Next,		next)
mapsimple_with_locking (Prev,		prev)
mapsimple_with_locking (Shuffle,	shuffle)
mapsimple_with_locking (Clear,		clear)

mapsimple_with_intret (Length,		    get_length)
mapsimple_with_intret (GetCurrent,	    get_current)
mapsimple_with_intret (Paused,		    is_paused)
mapsimple_with_intret (Crossfading,	    is_crossfading)
mapsimple_with_intret (LoopingSong,	    is_looping_song)
mapsimple_with_intret (LoopingPlaylist,	    is_looping_playlist)

// # # # # # # # # # # GetCorePlayer (path) # # # # # # # # # # # #
static PyObject *pyget_coreplayer (PyPlaylist_Object *self, PyObject *args)
{
    CorePlayer *cp;
    
    if (!PyArg_ParseTuple (args, ":get_coreplayer"))
	return NULL;
    
    Py_BEGIN_ALLOW_THREADS
    cp = self->cobject->GetCorePlayer ();
    Py_END_ALLOW_THREADS
    
    return (PyObject*)PyCorePlayer_New (cp, 0, self);
};
    
// # # # # # # # # # # PlayFile (playitem) # # # # # # # # # # # # 
static PyObject *pyplay_file (PyPlaylist_Object *self, PyObject *args)
{
    PyPlayItem_Object *pyplayitem;
    int rc;
    
    if (!PyArg_ParseTuple (args, "O:play_file", &pyplayitem))
	return NULL;

    if (!PyPlayItem_Check (pyplayitem)) {
	PyErr_SetString (PyExc_TypeError,
			 "first parameter must be object of PlayItem type");
	return NULL;
    }
 
    Py_BEGIN_ALLOW_THREADS
    rc = self->cobject->PlayFile (*pyplayitem->cobject);
    Py_END_ALLOW_THREADS
 
    if (!rc) {
	PyErr_SetString (PyAlsaplayerError, "error in play_file");
	return NULL;
    }
	
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # CanPlay (path) # # # # # # # # # # # # # # #
static PyObject *pycan_play (PyPlaylist_Object *self, PyObject *args)
{
    char *s;
    int rc;
    
    if (!PyArg_ParseTuple (args, "s:can_play", &s))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = self->cobject->CanPlay (s);
    Py_END_ALLOW_THREADS
    
    return Py_BuildValue ("i", rc);
};

// # # # # # # # # # # Play (n, locking)  # # # # # # # # # # # # #
static PyObject *pyplay (PyPlaylist_Object *self, PyObject *args)
{
    int n, locking = 0;
    
    if (!PyArg_ParseTuple (args, "i|i:play", &n, &locking))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    self->cobject->Play (n, locking);
    Py_END_ALLOW_THREADS
   
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # Insert list (paths, position)  # # # # # # # # #
static PyObject *pyinsert (PyPlaylist_Object *self, PyObject *args)
{
    PyObject *seq;
    int position;
    
    if (!PyArg_ParseTuple (args, "Oi:insert", &seq, &position))
	return NULL;

    // First parameter must be sequence
    if (!PySequence_Check (seq) || PyString_Check (seq)) {
	PyErr_SetString (PyExc_TypeError, "first parameter must be sequence");
	return NULL;
    }

    // Make C list
    std::vector<std::string> items;
    
    for (int i=0; i < PySequence_Length (seq); i++) {
	PyObject *item = PySequence_GetItem (seq, i);
	Py_DECREF (item);

	if (!PyString_Check (item)) {
	    PyErr_SetString (PyExc_TypeError,
			     "first parameter must be sequence of strings");
	    return NULL;
	}

	char *path = PyString_AsString (item);
	items.push_back (path);
    }
 
    Py_BEGIN_ALLOW_THREADS
    self->cobject->Insert (items, position);
    Py_END_ALLOW_THREADS
   
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # AddAndPlay (paths) # # # # # # # # # # # # # # #
static PyObject *pyadd_and_play (PyPlaylist_Object *self, PyObject *args)
{
    PyObject *seq;
    
    if (!PyArg_ParseTuple (args, "O:add_and_play", &seq))
	return NULL;

    // First parameter must be sequence
    if (!PySequence_Check (seq) || PyString_Check (seq)) {
	PyErr_SetString (PyExc_TypeError, "first parameter must be sequence");
	return NULL;
    }

    // Make C list
    std::vector<std::string> items;
    
    for (int i=0; i < PySequence_Length (seq); i++) {
	PyObject *item = PySequence_GetItem (seq, i);
	Py_DECREF (item);

	if (!PyString_Check (item)) {
	    PyErr_SetString (PyExc_TypeError,
			     "first parameter must be sequence of strings");
	    return NULL;
	}

	char *path = PyString_AsString (item);
	items.push_back (path);
    }
    
    Py_BEGIN_ALLOW_THREADS
    self->cobject->AddAndPlay (items);
    Py_END_ALLOW_THREADS
   
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # Remove (start, end) # # # # # # # # # # # # # 
static PyObject *pyremove (PyPlaylist_Object *self, PyObject *args)
{
    int start, end;
    
    if (!PyArg_ParseTuple (args, "ii:remove", &start, &end))
	return NULL;
 
    Py_BEGIN_ALLOW_THREADS
    int len = self->cobject->Length ();
    if (len != 0)  self->cobject->Remove (start, end);
    Py_END_ALLOW_THREADS
    
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # Save (file, format)  # # # # # # # # # # # #
static PyObject *pysave (PyPlaylist_Object *self, PyObject *args)
{
    char *filename;
    int format = PL_FORMAT_M3U;
    plist_result rc;
    
    if (!PyArg_ParseTuple (args, "s|i:save", &filename, &format))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = self->cobject->Save (filename, (plist_format)format);
    Py_END_ALLOW_THREADS

    if (rc == E_PL_BAD) {
	PyErr_SetString (PyPlaylistBadError, "bad status of save");
	return NULL;
    }
    
    if (rc == E_PL_DUBIOUS) {
	PyErr_SetString (PyPlaylistDubiousError, "dubious status of save");
	return NULL;
    }
    
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # Load (file, position, force)  # # # # # # # #
static PyObject *pyload (PyPlaylist_Object *self, PyObject *args)
{
    char *filename;
    int position, force;
    plist_result rc;
	
    if (!PyArg_ParseTuple (args, "sii:load", &filename, &position, &force))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = self->cobject->Load (filename, position, force);
    Py_END_ALLOW_THREADS

    if (rc == E_PL_BAD) {
	PyErr_SetString (PyPlaylistBadError, "bad status of load");
	return NULL;
    }
    
    if (rc == E_PL_DUBIOUS) {
	PyErr_SetString (PyPlaylistDubiousError, "dubious status of load");
	return NULL;
    }
    
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # Register (instance) # # # # # # # # # # # # # # #
static PyObject *pyregister (PyPlaylist_Object *self, PyObject *args)
{
    PyObject *instance;
    PyPlaylistInterfaceWrapper *wrapper;
   
    if (!PyArg_ParseTuple (args, "O:register", &instance))
	return NULL;

    // First parameter must be instance
    if (!PyInstance_Check (instance)) {
	PyErr_SetString (PyExc_TypeError, "first parameter must be instance");
	return NULL;
    }
   
    Py_BEGIN_ALLOW_THREADS
    wrapper = new PyPlaylistInterfaceWrapper (instance);
    self->cobject->Register (wrapper);
    Py_END_ALLOW_THREADS

    PyPlaylist_registration *reg = new PyPlaylist_registration ();
    reg->instance = instance;
    reg->wrapper = wrapper;
    self->registered->push_back (reg);
   
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # __internal__ () # # # # # # # # # # # # # # #
static PyObject *pyget_registered (PyPlaylist_Object *self, PyObject *args)
{
    if (!PyArg_ParseTuple (args, ":get_registered"))
	return NULL;
    
    // make tuple of instances
    PyObject *tuple = PyTuple_New (self->registered->size ());

    for (unsigned int i=0; i<self->registered->size (); i++) {
	PyObject *instance = (*self->registered) [i]->instance;

	Py_INCREF (instance);
	PyTuple_SetItem (tuple, i, instance);
    }

    return tuple;
}

// # # # # # # # # # # UnRegister (instance) # # # # # # # # # # # # # # #
static PyObject *pyunregister (PyPlaylist_Object *self, PyObject *args)
{
    PyObject *instance;
    PyPlaylistInterfaceWrapper *wrapper;
   
    if (!PyArg_ParseTuple (args, "O:unregister", &instance))
	return NULL;

    // First parameter must be instance
    if (!PyInstance_Check (instance)) {
	PyErr_SetString (PyExc_TypeError, "first parameter must be instance");
	return NULL;
    }
   
    // Search for wrapper
    for (unsigned int i=0; i<self->registered->size (); i++) {
	PyObject *inst = (*self->registered) [i]->instance;

	if (inst == instance) {
	    wrapper = (*self->registered) [i]->wrapper;
	    
	    Py_BEGIN_ALLOW_THREADS
	    self->cobject->UnRegister (wrapper);
	    Py_END_ALLOW_THREADS

	    delete wrapper;
	    delete (*self->registered) [i];
	    self->registered->erase (&(*self->registered) [i]);

	    break;
	}
    }

    Py_INCREF (Py_None);
    return Py_None;
};


// ##############################################################
// ##############################################################
static PyMethodDef methods[] = {
    {"play_file", (PyCFunction) pyplay_file, METH_VARARGS},
    {"stop", (PyCFunction) pystop, METH_VARARGS},
    {"can_play", (PyCFunction) pycan_play, METH_VARARGS},
    {"get_coreplayer", (PyCFunction) pyget_coreplayer, METH_VARARGS},
    {"play", (PyCFunction) pyplay, METH_VARARGS},
    {"get_length", (PyCFunction) pyget_length, METH_VARARGS},
    {"next", (PyCFunction) pynext, METH_VARARGS},
    {"prev", (PyCFunction) pyprev, METH_VARARGS},
    {"get_current", (PyCFunction) pyget_current, METH_VARARGS},
    {"insert", (PyCFunction) pyinsert, METH_VARARGS},
    {"add_and_play", (PyCFunction) pyadd_and_play, METH_VARARGS},
    {"remove", (PyCFunction) pyremove, METH_VARARGS},
    {"shuffle", (PyCFunction) pyshuffle, METH_VARARGS},
    {"clear", (PyCFunction) pyclear, METH_VARARGS},
    {"is_paused", (PyCFunction) pyis_paused, METH_VARARGS},
    {"pause", (PyCFunction) pypause, METH_VARARGS},
    {"unpause", (PyCFunction) pyunpause, METH_VARARGS},
    {"is_crossfading", (PyCFunction) pyis_crossfading, METH_VARARGS},
    {"crossfade", (PyCFunction) pycrossfade, METH_VARARGS},
    {"uncrossfade", (PyCFunction) pyuncrossfade, METH_VARARGS},
    {"is_looping_song", (PyCFunction) pyis_looping_song, METH_VARARGS},
    {"loop_song", (PyCFunction) pyloop_song, METH_VARARGS},
    {"unloop_song", (PyCFunction) pyunloop_song, METH_VARARGS},
    {"is_looping_playlist", (PyCFunction) pyis_looping_playlist, METH_VARARGS},
    {"loop_playlist", (PyCFunction) pyloop_playlist, METH_VARARGS},
    {"unloop_playlist", (PyCFunction) pyunloop_playlist, METH_VARARGS},
    {"save", (PyCFunction) pysave, METH_VARARGS},
    {"load", (PyCFunction) pyload, METH_VARARGS},
    {"register", (PyCFunction) pyregister, METH_VARARGS},
    {"get_registered", (PyCFunction) pyget_registered, METH_VARARGS},
    {"unregister", (PyCFunction) pyunregister, METH_VARARGS},
    {NULL, NULL}
};

static PyObject *getattr (PyObject *self, char *name)
{
    return Py_FindMethod (methods, self, name);
}

// DESTRUCTOR -- - -- -- ---- - - - - -- - - - - - -
static void dealloc (PyPlaylist_Object *self)
{
    // Unregister registered interfaces
    for (unsigned int i=0; i < self->registered->size (); i++) {
	PyPlaylistInterfaceWrapper *wrapper;
	wrapper = (*self->registered) [i]->wrapper;
	    
	Py_BEGIN_ALLOW_THREADS
	self->cobject->UnRegister (wrapper);
	Py_END_ALLOW_THREADS

	delete wrapper;
	delete (*self->registered) [i];
    }
    
    delete self->registered;

    if (self->delondealloc) {
        Py_BEGIN_ALLOW_THREADS
	delete self->cobject;
        Py_END_ALLOW_THREADS
    }
    
    PyObject_Del (self);
}

// Object creation function -  - - - - - - - - - - - - - - - - - - -
PyPlaylist_Object *PyPlaylist_New (Playlist *playlist, int delondealloc)
{
    PyPlaylist_Object *pyplaylist;

    pyplaylist = PyObject_New (PyPlaylist_Object, &PyPlaylist_Type);
    pyplaylist->cobject = playlist;
    pyplaylist->registered = new std::vector<PyPlaylist_registration *> ();
    pyplaylist->delondealloc = delondealloc;
    
    return pyplaylist;
}
