/*  pyplaylistinterface.cpp
 *  Copyright (C) 2002 Evgeny Chukreev <codedj@echo.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: pyplaylistinterface.cpp,v 1.4 2002/05/05 00:12:49 sjah Exp $
 *
*/ 

#include "pyplaylistinterface.h"
#include "pyplayitem.h"
#include "pydefs.h"

#define BLOCK_THREADS							\
    PyEval_AcquireLock ();						\
    PyThreadState *_tstate = PyThreadState_Swap (pycallback_tstate);

#define UNBLOCK_THREADS							\
    PyThreadState_Swap (_tstate);					\
    PyEval_ReleaseLock ();						\

PyPlaylistInterfaceWrapper::PyPlaylistInterfaceWrapper (PyObject *obj)
{
    pyobject = obj;
    Py_INCREF (pyobject);
    pthread_mutex_init (&lock, NULL);
}

PyPlaylistInterfaceWrapper::~PyPlaylistInterfaceWrapper ()
{
    Py_DECREF (pyobject);
}

// ------------------------- SET CURRENT -----------------------------
void PyPlaylistInterfaceWrapper::CbSetCurrent (unsigned pos)
{
    BLOCK_THREADS

    // get python callback
    if (PyObject_HasAttrString (pyobject, "cb_set_current")) {
	PyObject *cb = PyObject_GetAttrString (pyobject, "cb_set_current");
   
	// call 
	PyObject *arglist = Py_BuildValue ("(i)", pos);
	PyObject *result = PyEval_CallObject (cb, arglist);
	if (!result) {
	    PyErr_Print ();
	} else
	    Py_DECREF (result);

	// decrease references
	Py_DECREF (arglist);
	Py_DECREF (cb);
    }

    UNBLOCK_THREADS
}

// ------------------------- LOCK ----------------------------------
void PyPlaylistInterfaceWrapper::CbLock ()
{
    BLOCK_THREADS

    // get python callback
    if (PyObject_HasAttrString (pyobject, "cb_lock")) {
	PyObject *cb = PyObject_GetAttrString (pyobject, "cb_lock");

	// call 
	PyObject *arglist = Py_BuildValue ("()");
	PyObject *result = PyEval_CallObject (cb, arglist);
	if (!result) {
	    PyErr_Print ();
	} else
	   Py_DECREF (result);

	// decrease references
	Py_DECREF (arglist);
	Py_DECREF (cb);
    }

    UNBLOCK_THREADS
}

// ------------------------- UNLOCK -------------------------------
void PyPlaylistInterfaceWrapper::CbUnlock ()
{
    BLOCK_THREADS

    // get python callback
    if (PyObject_HasAttrString (pyobject, "cb_unlock")) {
	PyObject *cb = PyObject_GetAttrString (pyobject, "cb_unlock");

	// call 
	PyObject *arglist = Py_BuildValue ("()");
	PyObject *result = PyEval_CallObject (cb, arglist);
	if (!result) {
	    PyErr_Print ();
	} else
	    Py_DECREF (result);

	// decrease references
	Py_DECREF (arglist);
	Py_DECREF (cb);
    }

    UNBLOCK_THREADS
}

// ------------------------- INSERT ----------------------------------
void PyPlaylistInterfaceWrapper::CbInsert(std::vector<PlayItem> &items,
					  unsigned pos)
{
    BLOCK_THREADS
   
    // get python callback
    if (PyObject_HasAttrString (pyobject, "cb_insert")) {
	PyObject *cb = PyObject_GetAttrString (pyobject, "cb_insert");

	// make tuple of PlayItems
	PyObject *tuple = PyTuple_New (items.size ());
	for (unsigned int i=0; i<items.size (); i++) {
	    // Create new item becouse items from args is temporary
	    PlayItem *playitem = new PlayItem (items[i].filename);

	    // copy item
	    *playitem = items[i];
	    
	    // add to tuple
	    PyObject *item = (PyObject*)PyPlayItem_New (playitem, 1);
	    PyTuple_SetItem (tuple, i, item);
	}
	
	// call 
	PyObject *arglist = Py_BuildValue ("(Oi)", tuple, pos);
	PyObject *result = PyEval_CallObject (cb, arglist);
	if (!result) {
	    PyErr_Print ();
	} else
	    Py_DECREF (result);

	// decrease references
	Py_DECREF (arglist);
	Py_DECREF (tuple);
	Py_DECREF (cb);
    }

    UNBLOCK_THREADS
}

// ------------------------- REMOVE ----------------------------------
void PyPlaylistInterfaceWrapper::CbRemove(unsigned start, unsigned end)
{
    BLOCK_THREADS

    // get python callback
    if (PyObject_HasAttrString (pyobject, "cb_remove")) {
	PyObject *cb = PyObject_GetAttrString (pyobject, "cb_remove");
     
	// call 
	PyObject *arglist = Py_BuildValue ("(ii)", start, end);
	PyObject *result = PyEval_CallObject (cb, arglist);
	if (!result) {
	    PyErr_Print ();
	} else
	    Py_DECREF (result);
     
	// decrease references
	Py_DECREF (arglist);
	Py_DECREF (cb);
    }

    UNBLOCK_THREADS
}

// ------------------------- CLEAR ----------------------------------
void PyPlaylistInterfaceWrapper::CbClear()
{
    BLOCK_THREADS
    
    // get python callback
    if (PyObject_HasAttrString (pyobject, "cb_clear")) {
	PyObject *cb = PyObject_GetAttrString (pyobject, "cb_clear");

	// call 
	PyObject *arglist = Py_BuildValue ("()");
	PyObject *result = PyEval_CallObject (cb, arglist);
	if (!result) {
	    PyErr_Print ();
	} else
	    Py_DECREF (result);
	
	// decrease references
	Py_DECREF (arglist);
	Py_DECREF (cb);
    }

    UNBLOCK_THREADS
}

// ------------------------- UPDATED ----------------------------------
void PyPlaylistInterfaceWrapper::CbUpdated(PlayItem &item, unsigned pos)
{
    BLOCK_THREADS
   
    // get python callback
    if (PyObject_HasAttrString (pyobject, "cb_updated")) {
	PyObject *cb = PyObject_GetAttrString (pyobject, "cb_updated");

	// make PlayItems
	PlayItem *playitem = new PlayItem (item.filename);
	
	// copy item
	*playitem = item;

	PyObject *pyplayitem = (PyObject*)PyPlayItem_New (playitem, 1);
	
	// call 
	PyObject *arglist = Py_BuildValue ("(Oi)", pyplayitem, pos);
	PyObject *result = PyEval_CallObject (cb, arglist);
	if (!result) {
	    PyErr_Print ();
	} else
	    Py_DECREF (result);

	// decrease references
	Py_DECREF (arglist);
	Py_DECREF (pyplayitem);
	Py_DECREF (cb);
    }

    UNBLOCK_THREADS
}

